# xDevs.com Python 10V test for 3458A
# http://xdevs.com/guide/ni_gpib_rpi/
# G9330 10Kohm transfer test
#  3458A calibrated GPIB 3
#  K2002 for RTD GPIB 9
#  BME280 sensor I2C
# 
import os.path
import sys
import Gpib
import time
import ftplib
import numbers
import signal
#from Adafruit_BME280 import *
import thp_client

root_dir = '/root/xdevs_3458b/'
fileName4 = root_dir + 'mm_jrl1_5720a_100_3458_nplc100.csv'

cnt = 0
tread = 20
reflevel = 1.0
cmeas1 = 0.01
cmeas2 = -0.01
calc_res = 0
calc_resn =0
refhp = 200
k10v = 100000.0
tec_start = 24
tec_stop = 50
tec_step = 0.1
tec_time = 50
press = 0
tec_rtd = 0.3
temp = 18.0
res_rtd = 0.3
exttemp = 25.0
rh = 50.0
pascals = 100000.00
hectopascals = 1000.00
tset = 18.0
nv1 = 0.0
nv2 = 0.0

class Timeout():
  """Timeout class using ALARM signal"""
  class Timeout(Exception): pass

  def __init__(self, sec):
    self.sec = sec

  def __enter__(self):
    signal.signal(signal.SIGALRM, self.raise_timeout)
    signal.alarm(self.sec)

  def __exit__(self, *args):
    signal.alarm(0) # disable alarm

  def raise_timeout(self, *args):
    raise Timeout.Timeout()

class scpi_meter():
    temp = 38.5
    data = ""
    ppm = 0.0
    status_flag = 1
    temp_status_flag = 1
    global exttemp
    global rh
    global hectopascals
    global res_rtd

    def __init__(self,gpib,reflevel,name):
        self.gpib = gpib
        self.inst = Gpib.Gpib(0,self.gpib, timeout=90) # SCPI GPIB Address = self.gpib
        self.reflevel = reflevel
        self.name = name
        #self.init_inst()

    def init_inst(self):
        # Setup SCPI DMM
        self.inst.clear()
        self.inst.write("*RST")
        self.inst.write("*CLR")
        self.inst.write(":SYST:AZER:TYPE SYNC")
        self.inst.write(":SYST:LSYN:STAT ON")
        #self.inst.write(":sens:temp:tran rtd")      #select thermistor
        #self.inst.write(":sens:temp:rtd:type user")    #RTD from G9330
        #self.inst.write(":sens:temp:rtd:alph 0.00375") #RTD from G9330
        #self.inst.write(":sens:temp:rtd:beta 0.160")   #RTD from G9330
        #self.inst.write(":sens:temp:rtd:delt 1.605")   #RTD from G9330
        #self.inst.write(":sens:temp:rtd:rzer 1000")    #RTD from G9330
        #self.inst.write(":SENS:FUNC 'TEMP'")
        #self.inst.write(":SENS:TEMP:DIG 7")
        #self.inst.write(":SENS:TEMP:NPLC 10")
        self.inst.write(":SENS:FUNC 'FRES'")
        self.inst.write(":SENS:FRES:AVER:STAT OFF")
        self.inst.write(":SENS:FRES:RANG 20E3")
        self.inst.write(":SENS:FRES:DIG 8")
        self.inst.write(":SENS:FRES:NPLC 10")
        self.inst.write(":FORM:ELEM READ")

    def read_data(self,cmd):
        data_float = 0.0
        data_str = ""
        self.inst.write(cmd)
        try:
            with Timeout(90):
                data_str = self.inst.read()
        except Timeout.Timeout:
            print ("Timeout exception from dmm %s on read_data() inst.read()\n" % self.name)
            return (0,float(0))
        #print ("Reading from dmm %s = %s" % (self.name,data_str))
        try:
            data_float = float(data_str)
        except ValueError:
            print("Exception thrown by dmm %s on read_data() - ValueError = %s\n" % (self.name,data_str))
            return (0,float(0)) # Exception on float conversion, 0 = error
        return (1,data_float) # Good read, 1 = converted to float w/o exception

    def get_data(self):
	global tec_rtd
        self.status_flag,data = self.read_data("READ?")
        if (self.status_flag):
            self.data = data
            self.ppm = ((float(self.data) / self.reflevel)-1)*1E6
	    self.res_rtd = data
        tec_rtd = float(self.data)
        return self.data

    def get_data_fres(self):
	global k10v
        self.status_flag,data = self.read_data("READ?")
        if (self.status_flag):
            self.data = data
            self.ppm = ((float(self.data) / self.reflevel)-1)*1E6
	    self.res_rtd = data
        k10v = float(self.data)
        return self.data

    def get_data_status(self):
        return self.status_flag

    def write_data(self, fileHandle):
	res_rtd = self.data
	tempc = float(res_rtd)
        fileHandle.write(";%2.3f;\r\n" % tempc);
        #print time.strftime("%d/%m/%Y-%H:%M:%S;") + ("\033[1;31m[%8d]: %2.8f , dev %4.2f ppm,\033[1;39m  EXT_T:%3.2f , RH:%3.2f , Press:%4.2f hPa" % (cnt, float(self.data),float(self.ppm),float(exttemp),float(rh),float(hectopascals) ) )
        #fileHandle.write (time.strftime("%d/%m/%Y-%H:%M:%S;") + ("%16.8f;%16.8f;%3.1f;%3.2f;%3.2f;%4.2f;\r\n" % (float(self.data),float(self.reflevel),float(self.ppm),float(exttemp),float(rh),float(hectopascals) ) ))

    def print_ppm(self):
	res_rtd = self.data
        tempc = float(res_rtd)
        print ("%2.3f" % tempc)
	self.inst.write(":DISP:WIND2:TEXT:DATA \"%2.3f    \"" % tempc)
	#self.inst.write(":DISP:WIND2:TEXT:DATA \"  \"")

class nvhp():
    temp = 24.0
    data = ""
    ppm = 0.0
    status_flag = 1
    temp_status_flag = 1
    global exttemp
    global rh
    global hectopascals
    global res_rtd
    global tec_rtd

    def __init__(self,gpib,refhp,name):
        self.gpib = gpib
        self.inst = Gpib.Gpib(0,self.gpib, timeout=160) # 3458A GPIB Address = self.gpib
        self.refhp = refhp
        self.name = name
        self.init_inst()

    def init_inst(self):
        # Setup HP 34420A
        self.inst.clear()
	self.inst.write("*RST")
	self.inst.write("*CLS")
#        self.inst.write("CONF:VOLT:DC 0.01, MAX, (@FRONt1)")
#        self.inst.write("READ?")
        self.inst.write('SENS:FUNC "VOLT:DC"')
        self.inst.write("ROUT:TERM FRON1")
	self.inst.write("SENS:VOLT:DC:NPLC 100.000")
        self.inst.write("SENS1:VOLT:RANG 0.1")
        self.inst.write("DISP:STAT ON")
        self.inst.write("INP:FILT:LPAS:STAT OFF")
        self.inst.write("TRIG:DELAY 1")
        self.inst.write("TRIG:COUN 1")

    def read_data(self,cmd):
        data_float = 0.0
        data_str = ""
        self.inst.write(cmd)
    	time.sleep(0.1)
        try:
            with Timeout(160):
		time.sleep(0.1)
                data_str = self.inst.read()
        except Timeout.Timeout:
            print ("Timeout exception from dmm %s on read_data() inst.read()\n" % self.name)
            return (0,float(0))
        #print ("Reading from dmm %s = %s" % (self.name,data_str))
        try:
            data_float = float(data_str)
        except ValueError:
            print("Exception thrown by dmm %s on read_data() - ValueError = %s\n" % (self.name,data_str))
            return (0,float(0)) # Exception on float conversion, 0 = error
        return (1,data_float) # Good read, 1 = converted to float w/o exception

    def get_temp(self):
        self.inst.write("TARM SGL,1")
	time.sleep(0.1)        
	self.temp_status_flag,temp = self.read_data("TEMP?")
	time.sleep(0.1)
        if (self.temp_status_flag):
            self.temp = temp
        return self.temp

    def acaldcv(self):
	self.inst.write("ACAL DCV")
	time.sleep(140)
	return 0

    def get_temp_status(self):
        return self.temp_status_flag

    def get_data(self):
        self.status_flag,data = self.read_data("READ?")
	time.sleep(0.1)
	global calc_res
        if (self.status_flag):
            self.data = (data)
#            self.ppm = ((float(self.data) / self.refhp)-1)*1E6
            self.ppm = ((float(calc_res) / self.refhp)-1)*1E6
        return self.data

    def get_data_status(self):
        return self.status_flag

    def write_data(self,fileHandle):
	global tec_rtd
	global k10v
	global calc_res
	global press
        print time.strftime("%d/%m/%Y-%H:%M:%S;") + ("\033[1;36m[%8d]: %2.10f R , dev %4.4f ppm, T:%3.1f\033[1;39m , EXT_T:%3.2f , RH:%3.2f , Press:%4.2f hPa" % (cnt, calc_res ,float(self.ppm),float(self.temp),float(exttemp),float(rh),float(press) ) )
        fileHandle.write (time.strftime("%d/%m/%Y-%H:%M:%S;") + ("%.9f;%3.1f;%3.2f;%3.2f;%4.2f;\r\n" % (calc_res,float(self.temp),float(exttemp),float(rh),float(press) ) ))

    def print_ppm(self):
    #    self.inst.write("DISP OFF, \"%c%c%c%c%c %2.1f %c%c :)\"" % (0x1E,0xFF,0x11,0xFF,0x13,self.ppm,0x08,0x0C))
	global calc_res
	#self.inst.write("DISP OFF,%.10f" % calc_res)
        time.sleep(0.1)
#        self.inst.write("DISP OFF, \"Test\"")

class meter():
    temp = 24.0
    data = ""
    ppm = 0.0
    status_flag = 1
    temp_status_flag = 1
    global exttemp
    global rh
    global hectopascals
    global res_rtd
    global tec_rtd

    def __init__(self,gpib,refhp,name):
        self.gpib = gpib
        self.inst = Gpib.Gpib(0,self.gpib, timeout=160) # 3458A GPIB Address = self.gpib
        self.refhp = refhp
        self.name = name
        self.init_inst()

    def init_inst(self):
        # Setup HP 3458A
        self.inst.clear()
        self.inst.write("PRESET NORM")
        self.inst.write("OFORMAT ASCII")
        self.inst.write("DCV 0.1")
        self.inst.write("TARM HOLD")
        self.inst.write("TRIG AUTO")
        self.inst.write("NPLC 100")
        self.inst.write("AZERO ON")
        self.inst.write("OCOMP ON")
        self.inst.write("LFILTER ON")
        self.inst.write("NRDGS 1,AUTO")
        self.inst.write("MEM OFF")
        self.inst.write("END ALWAYS")
        self.inst.write("NDIG 9")
        self.inst.write("DELAY 0") # 2 second delay to mitigate OCOMP accuracy issue due DA

    def read_data(self,cmd):
        data_float = 0.0
        data_str = ""
        self.inst.write(cmd)
    	time.sleep(0.1)
        try:
            with Timeout(160):
		time.sleep(0.1)
                data_str = self.inst.read()
        except Timeout.Timeout:
            print ("Timeout exception from dmm %s on read_data() inst.read()\n" % self.name)
            return (0,float(0))
        #print ("Reading from dmm %s = %s" % (self.name,data_str))
        try:
            data_float = float(data_str)
        except ValueError:
            print("Exception thrown by dmm %s on read_data() - ValueError = %s\n" % (self.name,data_str))
            return (0,float(0)) # Exception on float conversion, 0 = error
        return (1,data_float) # Good read, 1 = converted to float w/o exception

    def get_temp(self):
        self.inst.write("TARM SGL,1")
	time.sleep(0.1)        
	self.temp_status_flag,temp = self.read_data("TEMP?")
	time.sleep(0.1)
        if (self.temp_status_flag):
            self.temp = temp
        return self.temp

    def acaldcv(self):
	self.inst.write("ACAL DCV")
	time.sleep(140)
	return 0

    def get_temp_status(self):
        return self.temp_status_flag

    def get_data(self):
        self.status_flag,data = self.read_data("TARM SGL,1")
	time.sleep(0.1)
	global calc_res
        if (self.status_flag):
            self.data = (data)
#            self.ppm = ((float(self.data) / self.refhp)-1)*1E6
            self.ppm = ((float(calc_res) / self.refhp)-1)*1E6
        return self.data

    def get_data_status(self):
        return self.status_flag

    def write_data(self,fileHandle):
	global tec_rtd
	global k10v
	global calc_res
        global calc_resn
	global press
	global cmeas1
	global cmeas2
	global nv1, nv2
	ppmn = ((float(calc_res) / self.refhp)-1)*1E6
        print time.strftime("%d/%m/%Y-%H:%M:%S;") + ("\033[1;36m[%8d]: I %2.10f %2.10f \033[1;35m%2.10f Rn\033[1;39m , dev %4.3f ppm \033[1;35m%4.3f ppm\033[1;39m, T:%3.1f\033[1;39m , EXT_T:%3.2f , RH:%3.2f , Press:%4.2f hPa" % (cnt, cmeas1, cmeas2 , calc_res, float(self.ppm), ppmn, float(self.temp),float(exttemp),float(rh),float(press) ) )
        fileHandle.write (time.strftime("%d/%m/%Y-%H:%M:%S;") + ("%.9f;%.9f;%.9f;%.9f;%.9f;%3.1f;%3.2f;%3.2f;%4.2f;\r\n" % (cmeas1,cmeas2,nv1,nv2,calc_res,float(self.temp),float(exttemp),float(rh),float(press) ) ))

    def print_ppm(self):
    #    self.inst.write("DISP OFF, \"%c%c%c%c%c %2.1f %c%c :)\"" % (0x1E,0xFF,0x11,0xFF,0x13,self.ppm,0x08,0x0C))
	global calc_resn
	#self.inst.write("DISP OFF,%.10f" % calc_resn)
        time.sleep(0.1)
#        self.inst.write("DISP OFF, \"Test\"")
        
## Read Temperature, Humidity, and Barometric Pressure from BME280
port = 10001
thp = thp_client.THP_socket('192.168.101.10', port)
error,exttemp,rh,press = thp.getTHP()

#def get_THP():
#    global exttemp
#    global rh
#    global pascals
#    global hectopascals
#    exttemp = sensor.read_temperature()
#    rh = sensor.read_humidity()
#    pascals = sensor.read_pressure()
#    hectopascals = pascals / 100
#    time.sleep(0.1)

# Check if file exists, if not create it and add header
def create_local_file(fileName):
    if (os.path.isfile(fileName) == False):
        with open(fileName, 'a') as o:
            o.write("date;dcip;dcin;nv1;nv2;hp34420a;temp;ext_temp;rh;pressure;\r\n")
            print ("file %s does not exist\r\n" % fileName) 
    else: 
        print ("file %s exists\r\n" % fileName)

mfc = Gpib.Gpib(0,1, timeout=30) # 4808
meter1 = meter(3,refhp,"3458A_00000") # Gpib 3
#hpnv = nvhp(12,refhp,"34420_00000") # Gpib 12

#cur1 = 0.001000024
#cur2 = -0.001000017

#meter1.inst.write ("ACAL DCV")
#time.sleep(140)

#mfc.write("S1R9F4O1=")
#mfc.write("S0R5F0O1=")
#mfc.write("M+0.10000000O1=")
#mfc.write("F2=")
#mfc.write("G0S0R0O0=")
#mfc.write("M-0.1000012O1=")
#mfc.write("S1A1O1=")

#meter2510 = tec_meter(25,reflevel,"2510_00000") # Gpib 25
#meter2002 = scpi_meter(9,reflevel,"2002_00000") # Gpib 10
#meter2002.init_inst()


#meter1.get_temp()
#meter1.inst.write("ACAL ALL")
#time.sleep(840)

#Setup temp/humidity/pressure sensor BME280
# OSAMPLE = oversampling mode
#sensor = BME280(mode=BME280_OSAMPLE_8)
#get_THP() # Read Temp, RH, Pressure from sensor

# Create file for 1st 3458A and write header if needed
create_local_file(fileName4)

cur1 = 0.1000000
cur2 = -0.10000000

temp = meter1.get_temp()
prevt = meter1.get_temp()

meter1.print_ppm()

#meter1.inst.write("ACAL ALL")
#time.sleep(860)

meas1 = 0
meas2 = 0

mfc.write("STBY")
mfc.write("OUT 100 mA")
mfc.write("RANGELCK ON")
mfc.write("OUT 0 mA")
mfc.write("OPER")
print "Zero curr"
time.sleep(3)

zerof = meter1.get_data()
time.sleep(3)
zerof = meter1.get_data()
time.sleep(3)
zerof = meter1.get_data()
print ("Zero voltage = %f uV" % (float(zerof) * 1e6) )

while cnt <= 500000:
    cnt+=1

    global calc_res
    global calc_resn
    global cmeas1
    global cmeas2
    global nv1, nv2
    #calc_res = meter1.get_data()

    error,exttemp,rh,press = thp.getTHP()
    
    #meter1.inst.write("DISP OFF,'%.8f %c-CALC'" % (calc_res, 0x0f))
    mfc.write("OUT %.6f A;OPER" % cur2) #Neg
    time.sleep(3)
    meas1 = meter1.get_data()
    time.sleep(3)
    meas1 += meter1.get_data()
    time.sleep(3)
    meas1 += meter1.get_data()
    meas1 = -(meas1 / 3)
    cmeas1 = meas1
    time.sleep(1)

    mfc.write("OUT %.6f A;OPER" % cur1) #Neg
    time.sleep(3)
    meas2 = meter1.get_data()
    time.sleep(3)
    meas2 += meter1.get_data()
    time.sleep(3)
    meas2 += meter1.get_data()
    meas2 = meas2 / 3
    cmeas2 = meas2
        
    #calc_res = ((meas2 - zerof) + (meas1 + zerof)) / (cur1 + (-cur2))
    #calc_resn = ((nv2 - zeron) + (nv1 + zeron)) / (cur1 + (-cur2))
    calc_res = ((meas2 - zerof) + (meas1 - zerof)) / 0.20
    #calc_resn = -((nv2) - (nv1)) / 0.04
    #calc_resn = meter1.get_data()

    with open(fileName4, 'a') as o1: # Open file handles for both files
        error,exttemp,rh,press = thp.getTHP()  # Read Temp, RH, Press from sensor
        tread = tread - 1
        if (tread == 0):
            tread = 120
            temp = meter1.get_temp()
	    if (temp > prevt + 2.9) or (temp < prevt - 2.9):
		meter1.acaldcv()
		prevt = temp

	#meter2002.get_data_fres()
        meter1.get_data()
	if (meter1.get_data_status()):
    	    meter1.write_data(o1)
            meter1.print_ppm()
    
        o1.close()
